@@ -1,5 +1,5 @@
 #!/usr/bin/perl -w
-my $RCS_Id = '$Id: BKM.pm,v 1.47 2006/03/05 20:57:29 jv Exp $ ';
+my $RCS_Id = '$Id$ ';
 
 package main;
 
@@ -13,8 +13,8 @@ package EB::Booking::BKM;
 # Author          : Johan Vromans
 # Created On      : Thu Jul  7 14:50:41 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Sun Mar  5 20:47:36 2006
-# Update Count    : 367
+# Last Modified On: Fri Oct  6 22:12:49 2006
+# Update Count    : 402
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -28,7 +28,7 @@ use warnings;
 
 use EB;
 use EB::DB;
-use EB::Finance;
+use EB::Format;
 use EB::Report::Journal;
 use base qw(EB::Booking);
 
@@ -43,7 +43,9 @@ sub perform {
     my $dagboek_type = $opts->{dagboek_type};
     my $totaal = $opts->{totaal};
     my $saldo = $opts->{saldo};
+    my $beginsaldo = $opts->{beginsaldo};
     my $does_btw = $dbh->does_btw;
+    my $verbose = $opts->{verbose};
 
     if ( defined($totaal) ) {
 	my $t = amount($totaal);
@@ -59,6 +61,13 @@ sub perform {
 	$saldo = $t;
     }
 
+    if ( defined($beginsaldo) ) {
+	my $t = amount($beginsaldo);
+	return "?".__x("Ongeldig beginsaldo: {saldo}", saldo => $beginsaldo)
+	  unless defined $t;
+	$beginsaldo = $t;
+    }
+
     my $bky = $self->{bky} ||= $opts->{boekjaar} || $dbh->adm("bky");
 
     my ($begin, $end);
@@ -82,19 +91,38 @@ sub perform {
 
     my $gdesc = shift(@$args);
 
+    my $bsk_nr = $self->bsk_nr($opts);
+    return unless defined($bsk_nr);
+
     my $nr = 1;
     my $bsk_id;
     my $gacct = $dbh->lookup($dagboek, qw(Dagboeken dbk_id dbk_acc_id));
 
-    print(__x("Huidig saldo: {bal}",
-	      bal => numfmt($dbh->lookup($gacct, qw(Accounts acc_id acc_balance)))), "\n")
-      if $gacct;
+    if ( $gacct ) {
+	my $vsaldo = saldo_for($dagboek, $bsk_nr-1, $bky);
+	if ( defined $beginsaldo ) {
+	    return "?".__x("Beginsaldo komt niet overeen met het eindsaldo van de voorgaande boeking",
+			   s1 => numfmt($beginsaldo), s2 => numfmt($vsaldo))."\n"
+			     if defined($vsaldo) && $vsaldo != $beginsaldo;
+	    print(__x("Beginsaldo: {bal}", bal => numfmt($beginsaldo)), "\n")
+	      if $verbose;
+	}
+	elsif ( defined $vsaldo ) {
+	    $beginsaldo = $vsaldo;
+	    print(__x("Saldo voorgaande boeking: {bal}", bal => numfmt($beginsaldo)), "\n")
+	      if $verbose;
+	}
+	else {
+	    $beginsaldo = $dbh->lookup($gacct, qw(Accounts acc_id acc_balance));
+	    print(__x("Huidig saldo: {bal}", bal => numfmt($beginsaldo)), "\n")
+	      if $verbose;
+	}
+    }
 
-    my $bsk_nr = $self->bsk_nr($opts);
+    $bsk_id = $dbh->get_sequence("boekstukken_bsk_id_seq");
     $dbh->sql_insert("Boekstukken",
-		     [qw(bsk_nr bsk_desc bsk_dbk_id bsk_date bsk_bky)],
-		     $bsk_nr, $gdesc, $dagboek, $date, $bky);
-    $bsk_id = $dbh->get_sequence("boekstukken_bsk_id_seq", "noincr");
+		     [qw(bsk_id bsk_nr bsk_desc bsk_dbk_id bsk_date bsk_bky)],
+		     $bsk_id, $bsk_nr, $gdesc, $dagboek, $date, $bky);
     my $tot = 0;
     my $did = 0;
     my $fail = 0;
@@ -124,7 +152,7 @@ sub perform {
 	      unless @$args >= 3;
 
 	    my ($desc, $amt, $acct) = splice(@$args, 0, 3);
-	    if ( $did++ || @$args || $opts->{verbose} ) {
+	    if ( $opts->{verbose} ) {
 		my $t = $desc;
 		$t = '"' . $desc . '"' if $t =~ /\s/;
 		warn(" "._T("boekstuk").": std $t $amt $acct\n");
@@ -209,7 +237,7 @@ sub perform {
 	    my ($btw_ink, $btw_verk);
 	    if ( $btw_id ) {
 		( $bsr_amount, $btw, $btw_ink, $btw_verk ) =
-		  @{EB::Finance::norm_btw($bsr_amount, $btw_id)};
+		  @{$self->norm_btw($bsr_amount, $btw_id)};
 		$amt = $bsr_amount - $btw;
 	    }
 	    $orig_amount = -$orig_amount;
@@ -261,7 +289,7 @@ sub perform {
 
 	    my ($rel, $amt) = splice(@$args, 0, 2);
 	    warn(" "._T("boekstuk").": $type $rel $amt\n")
-	      if $did++ || @$args || $opts->{verbose};
+	      if $verbose;
 
 	    my $oamt = $amt;
 	    $amt = amount($amt);
@@ -341,7 +369,6 @@ sub perform {
 				 bsr_rel_code bsr_paid)],
 			     $nr++, $dd, $bsk_id, "*".$bsk_desc, -$amt, 0,
 			     $type eq "deb" ? 1 : 2, $acct, 0, $bsr_rel, $bskid);
-	    my $id = $dbh->get_sequence("boekstukregels_bsr_id_seq", "noincr");
 	    $dbh->sql_exec("UPDATE Boekstukken".
 			   " SET bsk_open = bsk_open - ?".
 			   " WHERE bsk_id = ?",
@@ -361,10 +388,14 @@ sub perform {
     if ( $gacct ) {
 	warn("update $gacct with ".numfmt($tot)."\n") if $trace_updates;
 	$dbh->upd_account($gacct, $tot);
-	my $new = $dbh->lookup($gacct, qw(Accounts acc_id acc_balance));
-	print(__x("Nieuw saldo: {bal}", bal => numfmt($new)), "\n");
-	$dbh->sql_exec("UPDATE Boekstukken SET bsk_saldo = ? WHERE bsk_id = ?",
-		       $new, $bsk_id)->finish;
+#	my $new = $dbh->lookup($gacct, qw(Accounts acc_id acc_balance));
+	my $new = $beginsaldo + $tot;
+	print(__x("Nieuw saldo: {bal}", bal => numfmt($new)), "\n")
+	  if $verbose;
+	$dbh->sql_exec("UPDATE Boekstukken".
+		       " SET bsk_saldo = ?, bsk_isaldo = ?".
+		       " WHERE bsk_id = ?",
+		       $new, $beginsaldo, $bsk_id)->finish;
 	if ( defined $saldo ) {
 	    unless ( $saldo == $new ) {
 		warn("?".__x("Saldo {new} klopt niet met de vereiste waarde {act}",
@@ -374,9 +405,14 @@ sub perform {
 	}
 	if ( defined($totaal) and $tot != $totaal ) {
 	    $fail++;
-	    return "?"._T("Opdracht niet uitgevoerd.")." ".
-	      __x(" Boekstuk totaal is {act} in plaats van {exp}",
-		  act => numfmt($tot), exp => numfmt($totaal)) . ".";
+	    warn("?".__x(" Boekstuk totaal is {act} in plaats van {exp}",
+			 act => numfmt($tot), exp => numfmt($totaal)) . "\n");
+	}
+	my $isaldo = saldo_for($dagboek, $bsk_nr+1, $bky, "isaldo");
+	if ( defined($isaldo) and $isaldo != $new ) {
+	    $fail++;
+	    warn("?".__x("Saldo {new} klopt niet met beginsaldo eropvolgende boekstuk {isaldo}",
+			 new => numfmt($new), isaldo => numfmt($isaldo)) . "\n");
 	}
     }
     elsif ( $tot ) {
@@ -387,7 +423,7 @@ sub perform {
     $dbh->sql_exec("UPDATE Boekstukken SET bsk_amount = ? WHERE bsk_id = ?",
 		   $tot, $bsk_id)->finish;
 
-    $dbh->store_journal(EB::Finance::journalise($bsk_id));
+    $dbh->store_journal($self->journalise($bsk_id));
 
     if ( $opts->{journal} ) {
 	warn("?"._T("Dit overicht is ter referentie, de boeking is niet uitgevoerd!")."\n") if $fail;
@@ -404,7 +440,24 @@ sub perform {
     }
     $dbh->commit;
 
-    join(":", $dbh->lookup($dagboek, qw(Dagboeken dbk_id dbk_desc)), $bsk_nr);
+    # TODO -- need this to get a current booking.
+    $verbose || 1
+      ? join(":", $dbh->lookup($dagboek, qw(Dagboeken dbk_id dbk_desc)), $bsk_nr)
+	: "";
+}
+
+sub saldo_for {
+    my ($dbk, $nr, $bky, $ww) = (@_, "saldo");
+    my $sth = $dbh->sql_exec("SELECT bsk_$ww FROM Boekstukken".
+			     " WHERE bsk_dbk_id = ? AND bsk_nr = ?".
+			     " AND bsk_bky = ?",
+			     $dbk, $nr, $bky);
+    my $rr = $sth->fetchrow_arrayref;
+    $sth->finish;
+    if ( $rr && defined($rr->[0]) ) {
+	return $rr->[0];
+    }
+    return;
 }
 
 1;
