@@ -1,4 +1,4 @@
-my $RCS_Id = '$Id: Decode.pm,v 1.15 2006/03/07 08:55:06 jv Exp $ ';
+my $RCS_Id = '$Id$ ';
 
 package main;
 
@@ -11,8 +11,8 @@ package EB::Booking::Decode;
 # Author          : Johan Vromans
 # Created On      : Tue Sep 20 15:16:31 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Mon Mar  6 18:33:06 2006
-# Update Count    : 148
+# Last Modified On: Sun Oct  8 19:20:12 2006
+# Update Count    : 158
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -21,7 +21,8 @@ use strict;
 use warnings;
 
 use EB;
-use EB::Finance;
+use EB::Format;
+use EB::Booking;		# for norm_btw()
 
 sub new {
     return bless {};
@@ -55,7 +56,7 @@ sub decode {
     $bsk = $dbh->bskid($bsk);
 
     my $rr = $dbh->do("SELECT bsk_id, bsk_nr, bsk_desc, ".
-		      "bsk_dbk_id, bsk_date, bsk_amount, bsk_saldo, bsk_bky ".
+		      "bsk_dbk_id, bsk_date, bsk_amount, bsk_saldo, bsk_isaldo, bsk_bky ".
 		      ($dbver lt "001000002" ? ", bsk_paid" : ", bsk_open").
 		      " FROM Boekstukken".
 		      " WHERE bsk_id = ?", $bsk);
@@ -66,7 +67,7 @@ sub decode {
     }
 
     my ($bsk_id, $bsk_nr, $bsk_desc, $bsk_dbk_id,
-	$bsk_date, $bsk_amount, $bsk_saldo, $bsk_bky, $bsk_open) = @$rr;
+	$bsk_date, $bsk_amount, $bsk_saldo, $bsk_isaldo, $bsk_bky, $bsk_open) = @$rr;
 
     my $tot = 0;
     my ($dbktype, $acct, $dbk_desc) = @{$dbh->do("SELECT dbk_type, dbk_acc_id, dbk_desc".
@@ -81,7 +82,7 @@ sub decode {
 	    $cmd =~ s/[^[:alnum:]]/_/g;
 	    $cmd .= ":$bsk_bky" if $ex_bky;
 	    $cmd .= ":$bsk_nr" if $ex_bsknr;
-	    $cmd .= " $bsk_date ";
+	    $cmd .= " ".datefmt_full($bsk_date)." ";
 	    if ( $dbktype == DBKTYPE_INKOOP || $dbktype == DBKTYPE_VERKOOP ) {
 		$cmd .= $no_ivbskdesc ? "\"$rel_code\"" : "\"$bsk_desc\" \"$rel_code\"";
 	    }
@@ -92,13 +93,14 @@ sub decode {
 		$cmd .= " --totaal=" . numfmt_plain($dbktype == DBKTYPE_INKOOP ? 0-$bsk_amount : $bsk_amount)
 		  if $ex_tot && $acct;
 	    }
+	    $cmd .= " --beginsaldo=" . numfmt_plain($bsk_isaldo) if $ex_tot && defined $bsk_isaldo;
 	    $cmd .= " --saldo=" . numfmt_plain($bsk_saldo) if $ex_tot && defined $bsk_saldo;
 	}
 	else {
 	    $cmd = "Boekstuk $bsk_id, nr $bsk_nr, dagboek " .
 	      $dbh->lookup($bsk_dbk_id, qw(Dagboeken dbk_id dbk_desc =)).
 		"($bsk_dbk_id)".
-		  ", datum $bsk_date".
+		  ", ".datefmt_full(datum $bsk_date).
 		    ", ";
 	    if ( $dbktype == DBKTYPE_INKOOP || $dbktype == DBKTYPE_VERKOOP ) {
 		my ($rd, $rt) = @{$dbh->do("SELECT rel_desc,rel_debcrd".
@@ -119,21 +121,22 @@ sub decode {
 	}
     };
 
-    my $sth = $dbh->sql_exec("SELECT bsr_id, bsr_nr, bsr_date, ".
+    my $sth = $dbh->sql_exec("SELECT bsr_nr, bsr_date, ".
 			     "bsr_desc, bsr_amount, bsr_btw_id, bsr_btw_class, ".
 			     "bsr_type, bsr_acc_id, bsr_rel_code, bsr_paid ".
 			     " FROM Boekstukregels".
 			     " WHERE bsr_bsk_id = ?".
 			     " ORDER BY bsr_nr", $bsk);
 
-    unless ( $sth->rows ) {
+    $rr = $sth->fetchrow_arrayref;
+    unless ( $rr ) {
 	# Special case for boekstuk zonder boekstukregels.
 	$setup->(undef);
 	return $cmd;
     }
 
-    while ( $rr = $sth->fetchrow_arrayref ) {
-	my ($bsr_id, $bsr_nr, $bsr_date, $bsr_desc, $bsr_amount, $bsr_btw_id,
+    while ( $rr ) {
+	my ($bsr_nr, $bsr_date, $bsr_desc, $bsr_amount, $bsr_btw_id,
 	    $bsr_btw_class, $bsr_type, $bsr_acc_id, $bsr_rel_code, $bsr_paid) = @$rr;
 	if ( $bsr_nr == 1) {
 	    $setup->($bsr_rel_code);
@@ -149,7 +152,7 @@ sub decode {
 	my $dc = $bsr_amount >= 0 ? "debet" : "credit";
 	$dc = uc($dc) unless (($bsr_amount < 0) xor $rt);
 	$cmd .= join("",
-		     " Boekstukregel $bsr_id, nr $bsr_nr, datum $bsr_date, ",
+		     " Boekstukregel, nr $bsr_nr, datum $bsr_date, ",
 		     "\"$bsr_desc\"",
 		     ", type $bsr_type (", $bsr_types[$dbktype][$bsr_type], ")\n",
 		     "  ",
@@ -164,7 +167,7 @@ sub decode {
 	croak("INTERNAL ERROR: BTW/N id = $bsr_btw_id")
 	  if !($bsr_btw_class & BTWKLASSE_BTW_BIT) && $bsr_btw_id;
 
-	my $a = EB::Finance::norm_btw($bsr_amount, $bsr_btw_id);
+	my $a = EB::Booking::->norm_btw($bsr_amount, $bsr_btw_id);
 	$tot += $a->[0];
 
 	next unless $trail;
@@ -209,7 +212,7 @@ sub decode {
 		|| $dbktype == DBKTYPE_MEMORIAAL ) {
 	    $bsr_amount = -$bsr_amount;
 	    my $dd = "";
-	    $dd = " $bsr_date" unless $bsr_date eq $bsk_date;
+	    $dd = " ".datefmt_full($bsr_date) unless $bsr_date eq $bsk_date;
 	    if ( $bsr_type == 0 ) {
 		$cmd .= $single ? " " : " \\\n\t";
 		$cmd .= "std$dd \"$bsr_desc\" " .
@@ -242,7 +245,7 @@ sub decode {
 		}
 	    }
 	}
-
+	$rr = $sth->fetchrow_arrayref;
     }
     return ($cmd, $tot, $bsk_amount, $acct)
       if wantarray;
