@@ -1,10 +1,10 @@
-# Postgres.pm -- 
-# RCS Info        : $Id: Postgres.pm,v 1.8 2006/02/28 18:52:24 jv Exp $
+# Postgres.pm -- EekBoek driver for PostgreSQL dsatabase
+# RCS Info        : $Id$
 # Author          : Johan Vromans
 # Created On      : Tue Jan 24 10:43:00 2006
 # Last Modified By: Johan Vromans
-# Last Modified On: Tue Feb 28 19:47:18 2006
-# Update Count    : 90
+# Last Modified On: Wed Oct 11 14:27:31 2006
+# Update Count    : 163
 # Status          : Unknown, Use with caution!
 
 package main;
@@ -24,6 +24,7 @@ my $dataset;
 
 my $trace = $cfg->val(__PACKAGE__, "trace", 0);
 
+# API: type  type of driver
 sub type { "Postgres" }
 
 sub _dsn {
@@ -38,6 +39,7 @@ sub _dsn {
       : $dsn;
 }
 
+# API: create a new database, reuse an existing one if possible.
 sub create {
     my ($self, $dbname) = @_;
 
@@ -53,21 +55,24 @@ sub create {
     }
 
     croak("?INTERNAL ERROR: create db while connected") if $dbh;
-    my @ds = @{$self->list};
-    if ( grep { $_ eq $dbname } @ds ) {
-	$self->connect($dbname);
+    eval {
+	{
+	    local($SIG{__WARN__}) = sub {};
+	    $self->connect($dbname);
+	}
 	$dbh->{RaiseError} = 0;
 	$dbh->{PrintError} = 0;
 	$dbh->{AutoCommit} = 1;
 	$self->clear;
 	$self->disconnect;
-	return;
-    }
+    };
+    return unless $@;
+    die($@) if $cfg->unicode && $@ =~ /UNICODE/;
 
     $dbname =~ s/^(?!=eekboek_)/eekboek_/;
 
     my $sql = "CREATE DATABASE $dbname";
-    $sql .= " ENCODING 'LATIN1'";
+    $sql .= " ENCODING 'UNICODE'";
     for ( $cfg->val("database", "user", undef) ) {
 	next unless $_;
 	$sql .= " OWNER $_";
@@ -85,6 +90,7 @@ sub create {
 		err => $errstr)."\n");
 }
 
+# API: connect to an existing database.
 sub connect {
     my ($self, $dbname) = @_;
     croak("?INTERNAL ERROR: connect db without dataset name") unless $dbname;
@@ -101,9 +107,23 @@ sub connect {
       or die("?".__x("Database verbindingsprobleem: {err}",
 		     err => $DBI::errstr)."\n");
     $dataset = $dbname;
+    if ( $cfg->unicode ) {
+	my $enc = $dbh->selectall_arrayref("SHOW CLIENT_ENCODING")->[0]->[0];
+	if ( $enc !~ /^unicode|utf8$/i ) {
+	    warn("!".__x("Database {name} is niet in UTF-8 maar {enc}",
+			 name => $_[1], enc => $enc)."\n");
+	}
+	$dbh->do("SET CLIENT_ENCODING TO 'UNICODE'");
+	$dbh->{pg_enable_utf8} = 1;
+    }
+    else {
+	$dbh->do("SET CLIENT_ENCODING TO 'LATIN1'");
+	$dbh->{pg_enable_utf8} = 0;
+    }
     return $dbh;
 }
 
+# API: Disconnect from a database.
 sub disconnect {
     my ($self) = @_;
     return unless $dbh;
@@ -112,6 +132,10 @@ sub disconnect {
     undef $dataset;
 }
 
+# API: Setup whatever is needed.
+sub setup {
+}
+
 sub clear {
     my ($self) = @_;
     croak("?INTERNAL ERROR: clear db while not connected") unless $dbh;
@@ -119,49 +143,65 @@ sub clear {
     for my $tbl ( qw(Boekstukregels Journal Boekjaarbalans
 		     Metadata Standaardrekeningen Relaties
 		     Boekstukken Dagboeken Boekjaren Constants
-		     Accounts Btwtabel Verdichtingen) ) {
+		     Accounts Btwtabel Verdichtingen Taccounts) ) {
 	warn("+ DROP TABLE $tbl\n") if $trace;
-	$dbh->do("DROP TABLE $tbl");
+	eval { $dbh->do("DROP TABLE $tbl") };
     }
 
-    my $rr = $dbh->selectall_arrayref("SELECT relname".
-				      " FROM pg_class".
-				      " WHERE relkind = 'S'".
-				      ' AND relname LIKE \'bsk_%_seq\'');
-    foreach my $seq ( @$rr ) {
-	warn("+ DROP SEQUENCE $seq->[0]\n") if $trace;
-	$dbh->do("DROP SEQUENCE $seq->[0]");
-    }
+    eval {
+	my $rr = $dbh->selectall_arrayref("SELECT relname".
+					  " FROM pg_class".
+					  " WHERE relkind = 'S'".
+					  ' AND relname LIKE \'%bsk_%_seq\'');
+	foreach my $seq ( @$rr ) {
+	    warn("+ DROP SEQUENCE $seq->[0]\n") if $trace;
+	    eval { $dbh->do("DROP SEQUENCE $seq->[0]") };
+	}
+    };
     $dbh->commit unless $dbh->{AutoCommit};
+
 }
 
+# API: List available data sources.
 sub list {
     my @ds;
+
+    my $t;
+    local $ENV{PGHOST}   = $t if $t = $cfg->val(qw(database host), undef);
+    local $ENV{PGPORT}   = $t if $t = $cfg->val(qw(database port), undef);
+    local $ENV{DBI_USER} = $t if $t = $cfg->val("database", "user", undef);
+    local $ENV{DBI_PASS} = $t if $t = $cfg->val("database", "password", undef);
     eval {
-	local $ENV{PGUSER} = $cfg->val(qw(database user))
-	  if $cfg->val(qw(database user), undef);
-	local $ENV{PGPASS} = $cfg->val(qw(database password))
-	  if $cfg->val(qw(database password), undef);
-	local $ENV{PGHOST} = $cfg->val(qw(database host))
-	  if $cfg->val(qw(database host), undef);
-	local $ENV{PGPORT} = $cfg->val(qw(database port))
-	  if $cfg->val(qw(database port), undef);
 	@ds = DBI->data_sources("Pg");
-	die("Connect error:\n\t" . ($DBI::errstr||"")) if $DBI::errstr;
     };
+    # If the list cannot be established, @ds will be (undef).
+    return [] unless defined($ds[0]);
     [ map { $_ =~ s/^.*?dbname=eekboek_// and $_ } @ds ];
 }
 
+# API: Get a array ref with table names (lowcased).
+sub get_tables {
+    my $self = shift;
+    my @t;
+    foreach ( $dbh->tables ) {
+	next unless /^public\.(.+)/i;
+	push(@t, lc($1));
+    }
+    \@t;
+}
+
+################ Sequences ################
+
+# API: Get the next value for a sequence, incrementing it.
 sub get_sequence {
-    my ($self, $seq, $noinc) = @_;
+    my ($self, $seq) = @_;
     croak("?INTERNAL ERROR: get sequence while not connected") unless $dbh;
 
-    my $rr = $dbh->selectall_arrayref("SELECT ".
-				      ($noinc ? "currval" : "nextval").
-				      "('$seq')");
+    my $rr = $dbh->selectall_arrayref("SELECT nextval('$seq')");
     return ($rr && defined($rr->[0]) && defined($rr->[0]->[0])? $rr->[0]->[0] : undef);
 }
 
+# API: Set the next value for a sequence.
 sub set_sequence {
     my ($self, $seq, $value) = @_;
     croak("?INTERNAL ERROR: set sequence while not connected") unless $dbh;
@@ -172,6 +212,9 @@ sub set_sequence {
     $value;
 }
 
+################ Interactive SQL ################
+
+# API: Interactive SQL.
 sub isql {
     my ($self, @args) = @_;
 
@@ -181,13 +224,8 @@ sub isql {
 
     for ( $cfg->val("database", "user", undef) ) {
 	next unless $_;
-	push(@cmd, "-O", $_);
 	push(@cmd, "-U", $_);
     }
-#    for ( $cfg->val("database", "password", undef) ) {
-#	next unless $_;
-#	push(@cmd, "--password");
-#    }
     for ( $cfg->val("database", "host", undef) ) {
 	next unless $_;
 	push(@cmd, "-h", $_);
@@ -207,4 +245,34 @@ sub isql {
 
 }
 
+################ PostgreSQL Compatibility ################
+
+# API: feature  Can we?
+sub feature {
+    my $self = shift;
+    my $feat = lc(shift);
+
+    # Known features:
+    #
+    # pgcopy	F PostgreSQL fast input copying
+    # prepcache T Statement handles may be cached
+    # filter    C SQL filter routine
+    #
+    # Unknown/unsupported features may be ignored.
+
+    if ( $feat eq "pgcopy" ) {
+	return 1 if ($DBD::Pg::VERSION||0) >= 1.41;
+	warn("%"."Not using PostgreSQL fast load. DBD::Pg::VERSION = ",
+	     ($DBD::Pg::VERSION||0), ", needs 1.41 or later\n");
+	return;
+    }
+
+    return 1 if $feat eq "prepcache";
+
+    # Return false for all others.
+    return;
+}
+
+################ End PostgreSQL Compatibility ################
+
 1;
