@@ -1,5 +1,5 @@
 #!/usr/bin/perl -w
-my $RCS_Id = '$Id: Balres.pm,v 1.21 2006/03/06 15:46:43 jv Exp $ ';
+my $RCS_Id = '$Id$ ';
 
 package main;
 
@@ -13,8 +13,8 @@ package EB::Report::Balres;
 # Author          : Johan Vromans
 # Created On      : Sat Jun 11 13:44:43 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Mon Mar  6 11:02:14 2006
-# Update Count    : 353
+# Last Modified On: Thu May 25 18:59:18 2006
+# Update Count    : 366
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -26,7 +26,7 @@ use warnings;
 
 use EB;
 use EB::Report;
-use EB::Finance;
+use EB::Format;
 
 ################ Subroutines ################
 
@@ -59,6 +59,7 @@ sub perform {
     my ($self, $opts) = @_;
 
     my $balans = $opts->{balans};
+    my $opening = $opts->{opening};
     my $detail = $opts->{detail};
     $detail = $opts->{verdicht} ? 2 : -1 unless defined $detail;
     $opts->{detail} = $detail;
@@ -93,7 +94,13 @@ sub perform {
     if ( $balans < 0 ) {
 	my $date = $dbh->adm("begin");
 	$rep->start(_T("Openingsbalans"),
-		    __x("Datum: {date}", date => $now));
+		    __x("Datum: {date}", date => datefmt_full($now)));
+    }
+    elsif ( $opening ) {
+	my $date = $begin;
+	$rep->start(_T("Openingsbalans"),
+		    __x("Datum: {date}", date => datefmt_full($date)));
+	$table = EB::Report->GetTAccountsBal($date, 1);
     }
     else {
 	if ( $balans ) {
