@@ -1,21 +1,273 @@
 #!/usr/bin/perl
 
-my $RCS_Id = '$Id: Shell.pm,v 1.70 2006/03/10 12:51:24 jv Exp $ ';
+my $RCS_Id = '$Id$ ';
 
 # Author          : Johan Vromans
 # Created On      : Thu Jul  7 15:53:48 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Thu Mar  9 13:07:05 2006
-# Update Count    : 773
+# Last Modified On: Mon Oct 16 15:01:55 2006
+# Update Count    : 829
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
 
 package main;
 
+use strict;
+
+# Package or program libraries, if appropriate.
+# $LIBDIR = $ENV{'LIBDIR'} || '/usr/local/lib/sample';
+# use lib qw($LIBDIR);
+# require 'common.pl';
+
+# Package name.
+use EekBoek;
+my $my_package; BEGIN { $my_package = $EekBoek::PACKAGE }
+
+# Program name and version.
+my ($my_name, $my_version) = $RCS_Id =~ /: (.+),v ([\d.]+)/;
+# Tack '*' if it is not checked in into RCS.
+$my_version .= '*' if length('$Locker$ ') > 12;
+
+################ Configuration ################
+
 our $cfg;
 our $dbh;
 
+sub shell {
+
+# This will set up the config at 'use' time.
+use EB::Config $my_package;
+
+if ( @ARGV && ( $ARGV[0] eq '-P' || $ARGV[0] =~ /^--?printcfg$/ ) ) {
+    shift(@ARGV);
+    printconf();
+    exit;
+}
+
+################ Command line parameters ################
+
+use Getopt::Long 2.13;
+
+# Command line options.
+my $interactive = -t;
+my $command;
+my $echo;
+my $dataset;
+my $createdb;			# create database
+my $schema;			# initialise w/ schema
+my $confirm = 0;
+my $journal = 0;
+my $inexport;			# in/export
+my $inex_file;			# file voor in/export
+my $inex_dir;			# directory voor in/export
+my $verbose = 0;		# verbose processing
+my $bky;
+
+# Development options (not shown with -help).
+my $debug = 0;			# debugging
+my $trace = 0;			# trace (show process)
+my $test = 0;			# test mode.
+
+# Process command line options.
+app_options();
+
+# Post-processing.
+$trace |= ($debug || $test);
+
+################ Presets ################
+
+my $TMPDIR = $ENV{TMPDIR} || $ENV{TEMP} || '/usr/tmp';
+
+################ The Process ################
+
+use EB;
+#use base qw(EB::Shell);
+
+my $app = $my_package;
+my $userdir = glob("~/.".lc($app));
+mkdir($userdir) unless -d $userdir;
+
+$echo = "eb> " if $echo;
+
+$dataset ||= $cfg->val(qw(database name), undef);
+
+unless ( $dataset ) {
+    die("?"._T("Geen dataset opgegeven.".
+	       " Specificeer een dataset in de configuratiefile,".
+	       " of geef een dataset".
+	       " naam mee op de command line met \"--dataset=...\".").
+	"\n");
+}
+
+$cfg->newval(qw(database name), $dataset);
+
+use EB::DB;
+our $dbh = EB::DB->new(trace => $trace);
+
+if ( defined $inexport ) {
+    if ( $inexport ) {
+	$command = 1;
+	$createdb = 1;
+	@ARGV = qw(import --noclean);
+	push(@ARGV, "--file", $inex_file) if defined $inex_file;
+	push(@ARGV, "--dir", $inex_dir) if defined $inex_dir;
+    }
+    else {
+	$command = 1;
+	@ARGV = qw(export);
+	push(@ARGV, "--file", $inex_file) if defined $inex_file;
+	push(@ARGV, "--dir", $inex_dir) if defined $inex_dir;
+    }
+}
+
+if ( $createdb ) {
+    $dbh->createdb($dataset);
+    warn("%".__x("Lege dataset {db} is aangemaakt", db => $dataset)."\n");
+}
+
+if ( $schema ) {
+    require EB::Tools::Schema;
+    $dbh->connectdb(1);
+    EB::Tools::Schema->create($schema);
+}
+
+exit(0) if $command && !@ARGV;
+
+my $shell = EB::Shell->new
+  ({ HISTFILE	  => $userdir."/history",
+     command	  => $command,
+     interactive  => $interactive,
+     verbose	  => $verbose,
+     trace	  => $trace,
+     journal	  => $journal,
+     echo	  => $echo,
+     prompt	  => lc($app),
+     boekjaar	  => $bky,
+   });
+
+$| = 1;
+
+$shell->run;
+
+################ Subroutines ################
+
+sub printconf {
+    return unless @ARGV > 0;
+    my $sec = "general";
+    if ( !GetOptions(
+		     'section=s' => \$sec,
+		     '<>' => sub {
+			 my $conf = shift;
+			 my $sec = $sec;
+			 ($sec, $conf) = ($1, $2) if $conf =~ /^(.+?):(.+)/;
+			 my $val = $cfg->val($sec, $conf, undef);
+			 print STDOUT ($val) if defined $val;
+			 print STDOUT ("\n");
+		     }
+		    ) )
+    {
+	app_ident();
+	print STDERR __x(<<EndOfUsage, prog => $0);
+Gebruik: {prog} { --printcfg | -P } { [ --section=secname ] var ... } ...
+
+    Print de waarde van configuratie-variabelen.
+    Met --section=secname worden de eropvolgende variabelen
+    gezocht in sectie [secname].
+    Ook: secname:variabele.
+EndOfUsage
+    }
+}
+
+################ Subroutines ################
+
+sub app_options {
+    my $help = 0;		# handled locally
+    my $ident = 0;		# handled locally
+
+    # Process options, if any.
+    # Make sure defaults are set before returning!
+    return unless @ARGV > 0;
+
+    Getopt::Long::Configure(qw(no_ignore_case));
+
+    if ( !GetOptions(
+		     'command|c' => sub {
+			 $command = 1;
+			 die("!FINISH\n");
+		     },
+		     'import' => sub {
+			 $inexport = 1;
+		     },
+		     'export' => sub {
+			 $inexport = 0;
+		     },
+		     'createdb' => \$createdb,
+		     'define|D=s%' => sub {
+			 my ($opt, $key, $arg) = @_;
+			 if ( $key =~ /^(.+?)::?([^:]+)$/ ) {
+			     $cfg->newval($1, $2, $arg);
+			 }
+			 else {
+			     die(__x("Ongeldige aanduiding voor config setting: {arg}",
+				    arg => $key)."\n");
+			 }
+		     },
+		     'schema=s' => \$schema,
+		     'echo|e!'	=> \$echo,
+		     'ident'	=> \$ident,
+		     'journaal'	=> \$journal,
+		     'boekjaar=s'	=> \$bky,
+		     'verbose'	=> \$verbose,
+		     'db|dataset=s' => \$dataset,
+		     'dir=s'	=> \$inex_dir,
+		     'file=s'	=> \$inex_file,
+		     'trace'	=> \$trace,
+		     'help|?'	=> \$help,
+		     'debug'	=> \$debug,
+		    ) or $help )
+    {
+	app_usage(2);
+    }
+    app_usage(2) if @ARGV && !$command;
+    app_ident() if $ident;
+}
+
+sub app_ident {
+    print STDERR (__x("Dit is {pkg} [{name} {version}]",
+		      pkg     => $my_package,
+		      name    => $my_name,
+		      version => $my_version) . "\n");
+}
+
+sub app_usage {
+    my ($exit) = @_;
+    app_ident();
+    print STDERR __x(<<EndOfUsage, prog => $0);
+Gebruik: {prog} [options] [file ...]
+
+    --command  -c       voer de rest van de opdrachtregel uit als command
+    --echo  -e          toon ingelezen opdrachten
+    --journaal          toon de journaalregels na elke opdracht
+    --dataset=DB        specificeer database
+    --db=DB             specificeer database
+    --boekjaar=XXX	specificeer boekjaar
+    --createdb		maak nieuwe database aan
+    --schema=XXX        initialisser database met schema
+    --import            importeer een nieuwe administratie
+    --export            exporteer een administratie
+    --dir=XXX           directory voor im/export
+    --file=XXX          bestand voor im/export
+    --define=XXX -D     definieer configuratiesetting
+    --help		deze hulpboodschap
+    --ident		toon identificatie
+    --verbose		geef meer uitgebreide information
+EndOfUsage
+    exit $exit if defined $exit && $exit != 0;
+}
+
+}
+
 package EB::Shell;
 
 use strict;
@@ -29,7 +281,12 @@ sub new {
     $class = ref($class) || $class;
     my $opts = UNIVERSAL::isa($_[0], 'HASH') ? shift : { @_ };
 
-    _plug_cmds();
+    if ( $opts->{command} && $ARGV[0] eq "import" ) {
+	$dbh->connectdb(1);
+    }
+    else {
+	_plug_cmds();
+    }
 
     # User defined stuff.
     my $pkg = $cfg->val(qw(shell userdefs), undef);
@@ -120,11 +377,16 @@ sub eb_complete {
 	my $sth = $dbh->sql_exec("SELECT acc_id,acc_desc from Accounts".
 				 " WHERE acc_id LIKE ?".
 				 " ORDER BY acc_id", "$word%");
-	return () if $sth->rows == 0;
-	return ($sth->fetchrow_arrayref->[0]) if $sth->rows == 1;
+	my $rr = $sth->fetchrow_arrayref;
+	return () unless $rr;
+	my ($w, $d) = @$rr;
+	$rr = $sth->fetchrow_arrayref;
+	return ($w) unless $rr;
 	print STDERR ("\n");
-	while ( my $rr = $sth->fetchrow_arrayref ) {
+	printf STDERR ("%9d  %s\n", $w, $d);
+	while ( $rr ) {
 	    printf STDERR ("%9d  %s\n", @$rr);
+	    $rr = $sth->fetchrow_arrayref;
 	}
 	print STDERR ("$line");
 	return ();
@@ -139,14 +401,20 @@ sub eb_complete {
 				 " WHERE rel_code LIKE ?".
 				 " AND " . ($t eq "deb" ? "" : "NOT ") . "rel_debcrd".
 				 " ORDER BY rel_code", "$word%");
-	return () if $sth->rows == 0;
-	if ( $sth->rows == 1 && $word ne "" ) {
-	    $t = $sth->fetchrow_arrayref->[0];
-	    return ($t);
+	my $rr = $sth->fetchrow_arrayref;
+	return () unless $rr;
+
+	my ($w, $d) = @$rr;
+	$rr = $sth->fetchrow_arrayref;
+
+	if ( !$rr && $word ne "" ) {
+	    return ($w);
 	}
 	print STDERR ("\n");
-	while ( my $rr = $sth->fetchrow_arrayref ) {
+	printf STDERR ("  %s  %s\n", $w, $d);
+	while ( $rr  ) {
 	    printf STDERR ("  %s  %s\n", @$rr);
+	    $rr = $sth->fetchrow_arrayref;
 	}
 	print STDERR ("$line");
 	return ();
@@ -157,6 +425,7 @@ sub eb_complete {
 
 sub parseline {
     my ($self, $line) = @_;
+    $line =~ s/\\\s*$//;
     $line =~ s/;\s*$//;
     my ($cmd, $env, @args) = $self->SUPER::parseline($line);
 
@@ -171,10 +440,6 @@ sub parseline {
 
 use EB;
 
-# Standard options for report generating backends.
-my @outopts;
-INIT { @outopts = qw(html csv text output=s page=i) }
-
 # Plug in some commands dynamically.
 sub _plug_cmds {
     my $does_btw = $dbh->does_btw;
@@ -375,6 +640,7 @@ sub _add {
 		 d_boekjaar   => $bky || $dbh->adm("bky"),
 		 journal      => $self->{journal},
 		 totaal	      => undef,
+		 verbose      => $self->{verbose},
 		 confirm      => $self->{confirm},
 	       };
 
@@ -387,12 +653,12 @@ sub _add {
 		 'totaal=s',
 		 'confirm!',
 		 ( $dagboek_type == DBKTYPE_BANK
-		   || $dagboek_type == DBKTYPE_KAS ) ? ( 'saldo=s' ) : (),
+		   || $dagboek_type == DBKTYPE_KAS ) ? ( 'saldo=s', 'beginsaldo=s' ) : (),
 	       ], $opts);
 
     $opts->{boekjaar} = $opts->{d_boekjaar} unless defined $opts->{boekjaar};
     $bsk = $action->perform($args, $opts);
-    $bsk ? $bsk =~ /^\w+:\d+/ ? __x("Boekstuk: {bsk}", bsk => $bsk) : $bsk : "";
+    $bsk ? $bsk =~ /^\w+:\d+/ ? __x("Geboekt: {bsk}", bsk => $bsk) : $bsk : "";
 }
 
 ################ Reports ################
@@ -456,12 +722,18 @@ sub do_balans {
     parse_args(\@args,
 	       [ 'detail=i',
 		 'verdicht',
+		 'opening',
 		 'boekjaar=s',
 		 'per=s' => sub { date_arg($opts, @_) },
 		 EB::Report::GenBase->backend_options(EB::Report::Balres::, $opts),
 	       ], $opts);
     return unless argcnt(@args, 0);
 
+    if ( $opts->{opening} && $opts->{per} ) {
+	warn("?"._T("Openingsbalans kent geen einddatum")."\n");
+	return;
+    }
+
     EB::Report::Balres->new->balans($opts);
     undef;
 }
@@ -476,6 +748,7 @@ Opties:
   --detail=<n>                Verdicht, mate van detail <n> = 0, 1 of 2
   --per=<datum>               Selecteer einddatum
   --boekjaar=<code>           Selecteer boekjaar
+  --opening                   Toon openingsbalans
 
 Zie verder "help rapporten" voor algemene informatie over aan te maken
 rapporten.
@@ -906,7 +1179,7 @@ sub do_export {
     return unless
     parse_args(\@args,
 	       [ 'dir=s',
-		 'output=s',
+		 'file|output=s',
 		 'boekjaar=s',
 		 'single',
 		 'explicit',
@@ -914,12 +1187,12 @@ sub do_export {
 	       ], $opts)
       or goto &help_export;
 
-    if ( defined($opts->{dir}) && defined($opts->{output}) ) {
-	warn("?"._T("Opties --dir en --output sluiten elkaar uit")."\n");
+    if ( defined($opts->{dir}) && defined($opts->{file}) ) {
+	warn("?"._T("Opties --dir en --file sluiten elkaar uit")."\n");
 	return;
     }
-    if ( !defined($opts->{dir}) && !defined($opts->{output}) ) {
-	warn("?"._T("Specifieer --dir of --output")."\n");
+    if ( !defined($opts->{dir}) && !defined($opts->{file}) ) {
+	warn("?"._T("Specifieer --dir of --file")."\n");
 	return;
     }
 
@@ -940,32 +1213,34 @@ Exporteert de complete administratie.
 
 Opties:
 
-  --output=<bestand>          Selecteer uitvoerbestand
+  --file=<bestand>          Selecteer uitvoerbestand
   --dir=<directory>           Selecteer uitvoerdirectory
   --boekjaar=<code>           Selecteer boekjaar
 
-Er moet of een --output of een --dir optie worden opgegeven.
+Er moet of een --file of een --dir optie worden opgegeven.
 Zonder --boekjaar selectie wordt de gehele administratie gexporteerd.
+Eventueel bestaande files worden overschreven.
 EOS
 }
 
 sub do_import {
     my ($self, @args) = @_;
-    my $opts = { 
+    my $opts = { clean => 1,
 	       };
 
     return unless
     parse_args(\@args,
 	       [ 'dir=s',
 		 'file=s',
+		 'clean!',
 	       ], $opts);
 
-    if ( defined($opts->{dir}) && defined($opts->{output}) ) {
-	warn("?"._T("Opties --dir en --output sluiten elkaar uit")."\n");
+    if ( defined($opts->{dir}) && defined($opts->{file}) ) {
+	warn("?"._T("Opties --dir en --file sluiten elkaar uit")."\n");
 	return;
     }
-    if ( !defined($opts->{dir}) && !defined($opts->{output}) ) {
-	warn("?"._T("Specifieer --dir of --output")."\n");
+    if ( !defined($opts->{dir}) && !defined($opts->{file}) ) {
+	warn("?"._T("Specifieer --dir of --file")."\n");
 	return;
     }
 
@@ -1110,7 +1385,7 @@ sub do_verwijder {
 	    warn("?".$err."\n");
 	    return;
 	}
-	$cmd = EB::Booking::Decode->decode($id, { trail => 1, bsknr => 1, single => 1 });
+	$cmd = EB::Booking::Decode->decode($id, { boekjaar => $opts->{boekjaar}, trail => 1, bsknr => 1, single => 1 });
     }
     my $res = EB::Booking::Delete->new->perform($id, $opts);
     if ( $self->{interactive} && $res !~ /^[?!]/ ) {	# no error
