@@ -1,10 +1,10 @@
-# $Id: Opening.pm,v 1.23 2006/03/05 20:57:49 jv Exp $
+# $Id$
 
 # Author          : Johan Vromans
 # Created On      : Tue Aug 30 09:49:11 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Sun Mar  5 21:37:50 2006
-# Update Count    : 198
+# Last Modified On: Fri Oct  6 22:00:35 2006
+# Update Count    : 227
 # Status          : Unknown, Use with caution!
 
 package main;
@@ -17,7 +17,7 @@ package EB::Tools::Opening;
 
 use strict;
 use EB;
-use EB::Finance;
+use EB::Format;
 use EB::DB;
 
 # List of API methods (for the shell).
@@ -110,7 +110,15 @@ sub set_balans {
     my $anew;
     return __x("Ongeldig bedrag: {amount}", amount => $amt)."\n" unless defined($anew = amount($amt));
     $self->check_open(0);
-    push(@{$self->{o}->{balans}}, [$acct, $debcrd ? $anew : -$anew]);
+    $anew = -$anew unless $debcrd;
+    if ( exists($self->{o}->{balans}->{$acct}) ) {
+	my $e = $self->{o}->{balans}->{$acct};
+	$anew = -$anew if ($e->[0] xor $debcrd);
+	$self->{o}->{balans}->{$acct}->[1] += $anew;
+    }
+    else {
+	$self->{o}->{balans}->{$acct} = [ $debcrd, $anew ];
+    }
     "";
 }
 
@@ -140,7 +148,8 @@ sub set_relatie {
     return __x("Ongeldige datum: {date}", date => $date)."\n"
       unless $t;
     $date = $t;
-    return __x("Datum {date} valt niet vr het boekjaar", date => $date)."\n"
+    return __x("Datum {date} valt niet vr het boekjaar",
+	       date => datefmt_full($date))."\n"
       if $self->{o}->{begindatum} && $self->{o}->{begindatum} <= $1;
     $bky = substr($date, 0, 4) unless defined $bky;
 
@@ -220,34 +229,58 @@ sub open {
 	$adeb{0+$rr->[0]} = 0;
     }
 
-    if ( $o->{balanstotaal} ) {
+    if ( defined($o->{balanstotaal}) ) {
 	my $adeb;
 	my $acrd;
 	my $need_rel = 0;
-	if ( !$o->{balans} ) {
+	if ( !$o->{balans} && $o->{openingsbalans} ) {
 	    $fail++;
 	    warn(_T("De openingsbalans is nog niet opgegeven")."\n");
 	}
 	else {
-	    my $debet = $o->{balanstotaal};
-	    my $credit = -$debet;
-	    foreach my $b ( @{$o->{balans}} ) {
-		my ($acct, $amt) = @$b;
-		if ( $amt >= 0 ) {
-		    $debet -= $amt;
+	    # Boekhoudkundig rekenen.
+	    my $bdebet  = $o->{balanstotaal};
+	    my $bcredit = -$bdebet;
+	    # Rekenkundig rekenen.
+	    my $rcredit = $bcredit;
+	    my $rdebet  = $bdebet;
+	    while ( my ($acct, $e) = each(%{$o->{balans}}) ) {
+		my ($dc, $amt) = @$e;
+		# Rekenkundig rekenen.
+		if ( $dc ) {
+		    $rdebet -= $amt;
 		}
 		else {
-		    $credit -= $amt;
+		    $rcredit -= $amt;
 		}
-		$need_rel++, $adeb{$acct} = $amt if defined($adeb{$acct});
-		$need_rel++, $acrd{$acct} = $amt if defined($acrd{$acct});
+		# Boekhoudkundig rekenen.
+		if ( $amt >= 0 ) {
+		    $bdebet -= $amt;
+		}
+		else {
+		    $bcredit -= $amt;
+		}
+		$need_rel++, $adeb{$acct} += $amt if defined($adeb{$acct});
+		$need_rel++, $acrd{$acct} += $amt if defined($acrd{$acct});
+	    }
+	    if ( ($rdebet || $rcredit) && ($bdebet || $bcredit) ) {
+		$fail++;
+		warn(_T("De openingsbalans is niet correct!")."\n");
+		warn(__x("Opgegeven balanstotaal = {total}",
+			 total => numfmt($o->{balanstotaal}))."\n");
+		warn(__x("Rekenkundig residu debet = {rdeb}, credit = {rcrd}",
+			 rdeb => numfmt($rdebet),
+			 rcrd => numfmt(-$rcredit)).
+		     ($rdebet == -$rcredit
+		      ? __x(" (balanstotaal {total})", total => numfmt($o->{balanstotaal} - $rdebet))
+		      : ())."\n");
+		warn(__x("Boekhoudkundig residu debet = {rdeb}, credit = {rcrd}",
+			 rdeb => numfmt($bdebet),
+			 rcrd => numfmt(-$bcredit)).
+		     ($bdebet == -$bcredit
+		      ? __x(" (balanstotaal {total})", total => numfmt($o->{balanstotaal} - $bdebet))
+		      : ())."\n");
 	    }
-	    $fail++, warn(_T("De openingsbalans is niet in balans!")."\n".
-			  __x("Totaal = {total}, residu debet = {rdeb}, residu credit = {rcrd}",
-			      total => numfmt($o->{balanstotaal}),
-			      rdeb => numfmt($debet),
-			      rcrd => numfmt(-$credit))."\n")
-	      if $debet || $credit;
 
 	    # Helpful hints...
 	    $fail++, warn(_T("Er zijn geen openstaande posten opgegeven")."\n")
@@ -368,12 +401,12 @@ sub open {
 		   scalar(parse_date($o->{begindatum} . "-01-01", undef, -1)),
 		   BKY_PREVIOUS);
 
-    if ( $o->{balanstotaal} ) {
-	foreach my $b ( @{$o->{balans}} ) {
-	    my ($acct, $amt) = @$b;
+    if ( defined $o->{balanstotaal} ) {
+	while ( my ($acct, $e) = each(%{$o->{balans}}) ) {
+	    my ($dc, $amt) = @$e;
 	    $dbh->sql_exec("UPDATE Accounts".
-			   " SET acc_balance = ?,".
-			   "     acc_ibalance = ?".
+			   " SET acc_balance = acc_balance + ?,".
+			   "     acc_ibalance = acc_ibalance + ?".
 			   " WHERE acc_id = ?",
 			   $amt, $amt, $acct);
 	}
@@ -403,14 +436,14 @@ sub open {
 		$amt = -$amt;
 	    }
 
+	    my $bsk_id = $dbh->get_sequence("boekstukken_bsk_id_seq");
 	    $dbh->sql_insert("Boekstukken",
-			     [qw(bsk_nr bsk_desc bsk_dbk_id bsk_date bsk_bky bsk_open bsk_amount)],
-			     $nr, $desc, $dagboek, $date, $bky, $amt, $amt);
+			     [qw(bsk_id bsk_nr bsk_desc bsk_dbk_id bsk_date bsk_bky bsk_open bsk_amount)],
+			     $bsk_id, $nr, $desc, $dagboek, $date, $bky, $amt, $amt);
 	    $dbh->sql_insert("Boekstukregels",
 			     [qw(bsr_nr bsr_date bsr_bsk_id bsr_desc bsr_rel_code bsr_amount
 				 bsr_type bsr_btw_class)],
-			     1, $date,
-			     $dbh->get_sequence("boekstukken_bsk_id_seq", "noincr"),
+			     1, $date, $bsk_id,
 			     $desc, $code, 0-$amt, 9, 0);
 	}
 #	my $highest = $dbh->get_sequence("bsk_nr_0_seq") + 1;
