@@ -1,10 +1,10 @@
-my $RCS_Id = '$Id: Schema.pm,v 1.39 2006/03/05 20:58:08 jv Exp $ ';
+my $RCS_Id = '$Id$ ';
 
 # Author          : Johan Vromans
 # Created On      : Sun Aug 14 18:10:49 2005
 # Last Modified By: Johan Vromans
-# Last Modified On: Sun Mar  5 21:05:03 2006
-# Update Count    : 570
+# Last Modified On: Mon Oct 16 12:36:43 2006
+# Update Count    : 635
 # Status          : Unknown, Use with caution!
 
 ################ Common stuff ################
@@ -28,28 +28,20 @@ my $my_package = 'EekBoek';
 # Program name and version.
 my ($my_name, $my_version) = $RCS_Id =~ /: (.+).pl,v ([\d.]+)/;
 # Tack '*' if it is not checked in into RCS.
-$my_version .= '*' if length('$Locker:  $ ') > 12;
+$my_version .= '*' if length('$Locker$ ') > 12;
 
 ################ The Process ################
 
 use EB;
-use EB::Finance;
+use EB::Format;
 use EB::DB;
+use Encode;
 
 ################ Subroutines ################
 
-sub findlib {
-    my ($file) = @_;
-    foreach ( @INC ) {
-	return "$_/EB/$file" if -e "$_/EB/$file";
-    }
-    undef;
-}
-
 ################ Schema Loading ################
 
 my $schema;
-my $fh;
 
 sub create {
     shift;			# singleton class method
@@ -70,13 +62,19 @@ sub create {
     }
 
     die("?".__x("Onbekend schema: {schema}", schema => $name)."\n") unless $file;
-    open($fh, "<$file") or die("?".__x("Toegangsfout schema data: {err}", err => $!)."\n");
+    open(my $fh, "<$file") or die("?".__x("Toegangsfout schema data: {err}", err => $!)."\n");
     $schema = $name;
-    $dbh = EB::DB->new(trace => $trace) unless $sql;
-    load_schema();
+    _create(undef, sub { <$fh> });
     __x("Schema {schema} genitialiseerd", schema => $name);
 }
 
+sub _create {
+    shift;			# singleton class method
+    my ($rl) = @_;
+    $dbh = EB::DB->new(trace => $trace) unless $sql;
+    load_schema($rl);
+}
+
 my @hvdi;			# hoofdverdichtingen
 my @vdi;			# verdichtingen
 my $max_hvd;			# hoogste waarde voor hoofdverdichting
@@ -119,6 +117,7 @@ sub scan_dagboeken {
     error(__x("Dubbel: dagboek {dbk}", dbk => $id)."\n") if defined($dbk{$id});
 
     my $type;
+    my $dcsplit,
     my $rek = 0;
     my $extra;
     while ( $desc =~ /^(.+?)\s+:([^\s:]+)\s*$/ ) {
@@ -137,6 +136,9 @@ sub scan_dagboeken {
 	elsif ( $extra =~ m/^rek(?:ening)?=(\d+)$/i ) {
 	    $rek = $1;
 	}
+	elsif ( $extra =~ m/^dc$/i ) {
+	    $dcsplit = 1;
+	}
 	else {
 	    error(__x("Dagboek {id}: onbekende info \"{info}\"",
 		      id => $id, info => $extra)."\n");
@@ -145,12 +147,14 @@ sub scan_dagboeken {
 
     error(__x("Dagboek {id}: het :type ontbreekt", id => $id)."\n") unless defined($type);
     error(__x("Dagboek {id}: het :rekening nummer ontbreekt", id => $id)."\n")
-      if ( $type == DBKTYPE_KAS || $type == DBKTYPE_BANK ) and !$type;
+      if ( $type == DBKTYPE_KAS || $type == DBKTYPE_BANK ) && !$type;
+    error(__x("Dagboek {id}: :dc is alleen toegestaan voor Kas en Bankboeken", id => $id)."\n")
+      if $dcsplit && !( $type == DBKTYPE_KAS || $type == DBKTYPE_BANK );
 #    error(__x("Dagboek {id}: rekeningnummer enkel toegestaan voor Kas en Bankboeken", id => $id)."\n")
 #      if $rek && !($type == DBKTYPE_KAS || $type == DBKTYPE_BANK || $type == DBKTYPE_MEMORIAAL);
 
     $dbk{$id} = $dbkid;
-    $dbk[$dbkid] = [ $id, $desc, $type, $rek||undef ];
+    $dbk[$dbkid] = [ $id, $desc, $type, $dcsplit, $rek||undef ];
 }
 
 sub scan_btw {
@@ -335,15 +339,52 @@ sub scan_result {
 }
 
 sub load_schema {
+    my ($rl) = shift;
 
     init_vars();
     my $scanner;		# current scanner
     $max_hvd = 9;
     $max_vrd = 99;
+    my $uerr = 0;
+    my $unicode = $cfg->unicode;
 
     %std = map { $_ => 0 } qw(btw_ok btw_vh winst crd deb btw_il btw_vl btw_ih);
-    $fh ||= *ARGV;
-    while ( <$fh> ) {
+    while ( $_ = $rl->() ) {
+	if ( /^\# \s*
+	      content-type: \s*
+              text (?: \s* \/ \s* plain)? \s* ; \s*
+              charset \s* = \s* (\S+) \s* $/ix ) {
+	    my $charset = lc($1);
+	    if ( $charset =~ /^(?:latin[19]|iso-?8859[-.]15?)$/i ) {
+		$unicode = 0;
+		next;
+	    }
+	    if ( $charset =~ /^(?:unicode|utf-?8)$/i ) {
+		$unicode = 1;
+		next;
+	    }
+	}
+
+	if ( $unicode xor $cfg->unicode ) {
+	    my $s = $_;
+	    eval {
+		if ( $cfg->unicode ) {
+		    $_ = decode($unicode ? 'utf8' : 'latin1', $s, 1);
+		}
+		else {
+		    Encode::from_to($_, 'utf8', 'latin1', 1);
+		}
+	    };
+	    if ( $@ ) {
+		warn("?".__x("Geen geldige {cs} tekens in regel {line} van de invoer",
+			     cs => $unicode ? "UTF-8" : "Latin1",
+			     line => $.)."\n".$_."\n");
+		warn($@);
+		$fail++;
+		next;
+	    }
+	}
+
 	next if /^\s*#/;
 	next unless /\S/;
 
@@ -391,16 +432,16 @@ sub load_schema {
     my ($need_deb, $need_crd) = (0,0);
     foreach ( @dbk ) {
 	next unless defined($_); # sparse
-	my ($id, $desc, $type, $rek) = @$_;
+	my ($id, $desc, $type, $dc, $rek) = @$_;
 	next if defined($rek);
 	if ( $type == DBKTYPE_INKOOP ) {
 	    $need_crd++;
-	    $_->[3] = $std{"crd"};
+	    $_->[4] = $std{"crd"};
 	    #### Verify that it's a C acct.
 	}
 	elsif ( $type == DBKTYPE_VERKOOP ) {
 	    $need_deb++;
-	    $_->[3] = $std{"deb"};
+	    $_->[4] = $std{"deb"};
 	    #### Verify that it's a D acct.
 	}
 	elsif ( $type != DBKTYPE_MEMORIAAL ) {
@@ -467,7 +508,7 @@ sub _tsv {
 sub sql_eekboek {
     my $f = findlib("schema/eekboek.sql");
     open (my $fh, '<', $f)
-      or die("?"._T("Installatiefout -- geen schema")."\n");
+      or die("?"._T("Installatiefout -- geen database schema")."\n");
 
     local $/;
     my $sql = <$fh>;
@@ -570,13 +611,13 @@ ESQL
 sub sql_dbk {
     my $out = <<ESQL;
 -- Dagboeken
-COPY Dagboeken (dbk_id, dbk_desc, dbk_type, dbk_acc_id) FROM stdin;
+COPY Dagboeken (dbk_id, dbk_desc, dbk_type, dbk_dcsplit, dbk_acc_id) FROM stdin;
 ESQL
 
     foreach ( @dbk ) {
 	next unless defined;
-	$_->[3] ||= $std{deb} if $_->[2] == DBKTYPE_VERKOOP;
-	$_->[3] ||= $std{crd} if $_->[2] == DBKTYPE_INKOOP;
+	$_->[4] ||= $std{deb} if $_->[2] == DBKTYPE_VERKOOP;
+	$_->[4] ||= $std{crd} if $_->[2] == DBKTYPE_INKOOP;
 	$out .= join("\t",
 		     map { defined($_) ? $_ : "\\N" } @$_).
 		       "\n";
@@ -619,6 +660,7 @@ sub dump_sql {
 }
 
 my %kopp;
+my $fh;
 
 sub dump_schema {
     my ($self, $fh) = @_;
@@ -631,7 +673,8 @@ sub dump_schema {
 	  "# Aangemaakt door ", __PACKAGE__, " $my_version");
     my @t = localtime(time);
     printf {$fh} ("op %02d-%02d-%04d %02d:%02d:%02d\n", $t[3], 1+$t[4], 1900+$t[5], @t[2,1,0]);
-
+    printf {$fh} ("# Content-Type: text/plain; charset = %s\n",
+		  $cfg->unicode ? "UTF-8" : "ISO-8859.1");
     print {$fh}  <<EOD;
 
 # Dit bestand definiert alle vaste gegevens van een administratie of
@@ -737,6 +780,9 @@ print {$fh}  <<EOD;
 # type het is. Voor dagboeken van het type Kas en Bank moet een
 # tegenrekening worden opgegeven, voor de overige dagboeken mag een
 # tegenrekening worden opgegeven.
+# De optie :dc kan worden gebruikt om aan te geven dat het journaal
+# voor dit dagboek de boekstuktotalen in gescheiden debet en credit
+# moet tonen.
 EOD
 
     dump_dbk($fh);			# Dagboeken
@@ -882,16 +928,18 @@ sub dump_btw {
 sub dump_dbk {
     my $fh = shift;
     print {$fh} ("\nDagboeken\n\n");
-    my $sth = $dbh->sql_exec("SELECT dbk_id, dbk_desc, dbk_type, dbk_acc_id".
+    my $sth = $dbh->sql_exec("SELECT dbk_id, dbk_desc, dbk_type, dbk_dcsplit, dbk_acc_id".
 			     " FROM Dagboeken".
 			     " ORDER BY dbk_id");
     while ( my $rr = $sth->fetchrow_arrayref ) {
-	my ($id, $desc, $type, $acc_id) = @$rr;
+	my ($id, $desc, $type, $dc, $acc_id) = @$rr;
 	$acc_id = 0 if $type == DBKTYPE_INKOOP  && $dbh->std_acc("crd", 0) == $acc_id;
 	$acc_id = 0 if $type == DBKTYPE_VERKOOP && $dbh->std_acc("deb", 0) == $acc_id;
 	my $t = sprintf("  %-4s  %-20s  :type=%-10s %s",
 			$id, $desc, lc(DBKTYPES->[$type]),
-			($acc_id ? ":rekening=$acc_id" : ""));
+			($acc_id ? ":rekening=$acc_id" : "").
+			($dc ? " :dc" : ""),
+		       );
 	$t =~ s/\s+$//;
 	print {$fh} ($t, "\n");
     }
